package org.ow2.petals.databinding.jaxb;

import java.util.HashMap;
import java.util.Map;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

import org.ow2.petals.databinding.jaxb.tools.DataBindingConstants;


/**
 * Registry to save the instances used to Marshall/UnMarshall 
 * operations messages
 * 
 * @author Mohammed EL JAI
 *
 */
public class Registry {
    
    private Map<String, MarshallingObjects> marshallingObjects;
    
    public Registry() {
        this.setMarshallingObjects(new HashMap<String, MarshallingObjects>());
    }

    public void setMarshallingObjects(Map<String, MarshallingObjects> marshallingObjects) {
        this.marshallingObjects = marshallingObjects;
    }

    public Map<String, MarshallingObjects> getMarshallingObjects() {
        return marshallingObjects;
    }
    
    public void addEntry(String nameSpace, String packageName, ClassLoader jaxbClassLoader) throws Exception {
        
        MarshallingObjects marshallingObjects = new MarshallingObjects();
        
        JAXBContext jaxbContext = JAXBContext.newInstance(packageName, jaxbClassLoader);
        
        Class<?> jaxbFactoryClass = 
            jaxbClassLoader.loadClass(packageName + "." + DataBindingConstants.JAXB_FACTORY_CLASS_NAME); 
        marshallingObjects.setJaxbFactoryClass(jaxbFactoryClass);
        
        Marshaller marshaller = jaxbContext.createMarshaller();
        marshallingObjects.setMarshaller(marshaller);
        
        Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();
        marshallingObjects.setUnmarshaller(unmarshaller);
        
        this.getMarshallingObjects().put(nameSpace, marshallingObjects);
                
    }
    
    public Marshaller getMarshaller(String nameSpace) {
        return marshallingObjects.get(nameSpace).getMarshaller();
    }
    
    public Unmarshaller getUnmarshaller(String nameSpace) {
        return  marshallingObjects.get(nameSpace).getUnmarshaller();
    }

    public Object getJaxbFactoryInstance(String nameSpace) {
        return marshallingObjects.get(nameSpace).getJaxbFactoryInstnace();
    }

  



    private class MarshallingObjects {
        
        /** **/
        private Marshaller marshaller;
        
        /** **/
        private Unmarshaller unmarshaller;
        
        /** **/
        private Class<?> jaxbFactoryClass; 
        
        private Object jaxbFactoryInstnace;
        
        public Marshaller getMarshaller() {
            return marshaller;
        }

        public void setMarshaller(Marshaller marshaller) {
            this.marshaller = marshaller;
        }

        public Unmarshaller getUnmarshaller() {
            return unmarshaller;
        }

        public void setUnmarshaller(Unmarshaller unmarshaller) {
            this.unmarshaller = unmarshaller;
        }

        @SuppressWarnings("unused")
        public Class<?> getJaxbFactoryClass() {
            return jaxbFactoryClass;
        }

        public void setJaxbFactoryClass(Class<?> jaxbFactoryClass) {
            try {
                this.setJaxbFactoryInstnace(jaxbFactoryClass.newInstance());
            } catch (InstantiationException e) {
            } catch (IllegalAccessException e) {
            }
            this.jaxbFactoryClass = jaxbFactoryClass;
        }

        public void setJaxbFactoryInstnace(Object jaxbFactoryInstnace) {
            this.jaxbFactoryInstnace = jaxbFactoryInstnace;
        }

        public Object getJaxbFactoryInstnace() {
            return jaxbFactoryInstnace;
        }
        
    }
     

}
